@@ -699,7 +699,9 @@ void TestJSONRPC::enableDisableNotifications_legacy()
 
     QStringList expectedNamespaces;
     if (enabled == "true") {
-        expectedNamespaces << "NetworkManager" << "Integrations" << "System" << "Rules" << "Logging" << "Tags" << "AppData" << "JSONRPC" << "Configuration" << "Scripts" << "Users" << "Zigbee" << "ZWave" << "ModbusRtu";
+        expectedNamespaces << "NetworkManager" << "Integrations" << "System" << "Rules" << "Logging" << "Tags"
+                           << "AppData" << "JSONRPC" << "Configuration" << "Scripts" << "Users" << "Zigbee"
+                           << "ZWave" << "ModbusRtu" << "Debug";
     }
     std::sort(expectedNamespaces.begin(), expectedNamespaces.end());
 
